require 'kyotocabinet'

module Ygg
  module SimpleService
    module Email
      class Domain < ActiveResource::Hel::Resource
        def self.cabinet
          @cabinet ||= begin
            path = Tomte.config[:mailserver][:db]
            raise StandardError.new("missing database path") unless path.present?
            
            path = Pathname.new(path)
            path = MailServerAgent.root.join(path) unless path.absolute?
            path.dirname.mkpath unless path.dirname.exist?

            db = KyotoCabinet::DB.new
            result = db.open(path.to_s, KyotoCabinet::DB::OWRITER | KyotoCabinet::DB::OCREATE)
            raise StandardError.new("cannot open cache database #{path.to_s}") unless result
            db
          end
        end

        def self.get_stored
          stored = []
          # #each swallows exceptions
          ex = nil
          cabinet.each do | key, data |
            next unless key =~ /\A#{self.name}/
            data = JSON.parse(data)
            stored << new(data, true)
          rescue 
            # #each swallows exceptions
            ex = $!
          end
          # #each swallows exceptions
          raise ex if ex
          stored
        end

        def self.provision!
          logger.debug "rebuilding amavis config"
          amavis_config = []
          path = Tomte.config[:mailserver][:private_keys_path]
          raise StandardError.new("missing private keys path") unless path.present?
          
          path = Pathname.new(path)
          path = MailServerAgent.root.join(path) unless path.absolute?
          
          # store private keys in a subdirectory to make the process almost atomic
          tmppath = path.join('private_keys.tmp')
          # delete old tempdir if exists
          tmppath.rmtree if tmppath.exist?
          tmppath.mkpath 

          bckpath = path.join('private_keys.old')

          path = path.join('private_keys')
          path.mkpath unless path.exist?

          get_stored.each do | record |
            dk = record.dkim_key || {}
            if dk[:enabled]
              key_filename = "#{record.domain}.pem"
              options = ''

              opts = []
              opts << "t=>'#{dk[:flags]}'" if dk[:flags].present?
              opts << "h=>'#{dk[:hash]}'" if dk[:hash].present?
              
              options = ", #{opts.join(', ')}" if opts.any?
              amavis_config << "dkim_key('#{record.domain}', '#{dk[:selector]}', '#{path.join(key_filename)}'#{options});"
              
              logger.debug "-> write private key #{path.join(key_filename).to_s}"
              tmpfile = tmppath.join(key_filename)
              tmpfile.open('w+') { |f| f.write dk[:private_key] }
            end
          end

          cfgfile = Tomte.config[:mailserver][:amavis_config]
          raise StandardError.new("missing amavis config path") unless cfgfile.present?
          cfgfile = Pathname.new(cfgfile)
          cfgfile = MailServerAgent.root.join(cfgfile) unless cfgfile.absolute?

          bckcfgfile = Pathname.new("#{cfgfile.to_s}.bck")

          bckpath.rmtree if bckpath.exist?
          cfgfile.rename(bckcfgfile) if cfgfile.exist?
          
          logger.debug "-> write amavis config #{cfgfile.to_s}"
          cfg = "use strict;\n#{amavis_config.join("\n")}\n1;"
          cfgfile.open('w+') {|f| f.write cfg }
          path.rename(bckpath) if path.exist?
          tmppath.rename(path)
          
          logger.debug "-> restart amavis: command '#{Tomte.config[:mailserver][:amavis_reload]}'"
          logger.debug `#{Tomte.config[:mailserver][:amavis_reload]}`
        end

        self.load_remote_schema = true
        self.timeout = 120

        def cabinet_key
          "#{self.class.name}##{self.id}"
        end

        def store!
          data = to_json
          self.class.cabinet.set(cabinet_key, data)
        end

        def get_stored
          data = self.class.cabinet.get(cabinet_key)
          stored = nil
          if data
            data = JSON.parse(data)
            stored = self.class.new(data, true)
          end
          stored
        end

        def delete_stored!
          self.class.cabinet.remove(cabinet_key)
        end

        def changed?
          old = get_stored
          changed = old.blank?
          ok = old && old.dkim_key || {}
          nk = self.dkim_key || {}
          changed = ok[:enabled]     != nk[:enabled]     unless changed
          changed = ok[:private_key] != nk[:private_key] unless changed
          changed = ok[:hash]        != nk[:hash]        unless changed
          changed = ok[:flags]       != nk[:flags]       unless changed
          changed
        end
      end
    end
  end
end