
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require 'mailserver_agent'

module Tomte::Agent
  class MailServerAgent < Tomte::Agent::Async

    DEFAULT_MAIL_AGENT_OPTIONS = {
    }

    read_configuration 'mailserver_agent.yml', :path => :application, :queue => :before
    read_configuration 'mailserver_agent.yml', :path => :system
    read_configuration 'config/agent.yml', :path => :cwd
    read_configuration 'mailserver_agent.yml', :path => :cwd

    caption   "Mail Agent"
    help_text "Whohoho!"

    base_configuration :mailserver, DEFAULT_MAIL_AGENT_OPTIONS

    has_worker :mailserver, :class => 'Tomte::Workers::MailServerWorkerPubsub', :file => 'tomte/workers/mailserver_worker_pubsub.rb'

    def initialize(*args)
      super

      setup config
    end

    private

    def setup(options = {})
      log.debug '+ Init: setup ActiveRecord'
      #ActiveRecord::Base.logger = log
      #::MailAgent::Models::Email::Base.establish_connection config[:database][:email]

      if config[:raven] && config[:raven][:dsn]
        require 'raven'
        Raven.configure do | c |
          c.dsn = config[:raven][:dsn]
        end
      end

      log.debug '+ Init: setup ActiveResource::Hel'
      ActiveResource::Hel::Resource.configurations = {
        hel: {
          site: config[:hel][:host],
          identity: config[:hel][:username],
          secret: config[:hel][:password]
        }
      }
      ActiveResource::Hel::Resource.ssl_options = {verify_mode: OpenSSL::SSL::VERIFY_NONE} if ENV['SSL_VERIFY_NONE']
      ActiveResource::Hel::Resource.use_configuration :hel
      ActiveResource::Hel::Resource.logger = log
    end

  end
end
