module Tomte::Workers
  class MailServerWorkerPubsub < Tomte::Worker

    bus :service, :enveloper => Tomte::Protocol::Enveloper::Generic do |ep|
      consumer = ep.consumer :queue => "ygg.email.events.#{Tomte.config[:local][:host]}", :ack => false
      consumer.connect! do |c|
        c.declare_exchange('ygg.email.events', :topic, :auto_delete => true) do
          c.bind('ygg.email.events')
          c.consume { log.info "ready to receive messages on ygg.email.events" }
        end

        consumer.prefetch 1
        c.consume { log.info "ready to receive messages" }

        log.debug '+ Sync database'
        @sync.sync
        @sync = nil
      end

      consumer.on_delivery do |metadata, message|
        log.debug "received message"
        log.debug message.ai
        ygg_obj = load_ygg_obj message
        case ygg_obj
        when Ygg::SimpleService::Email::Domain
          case message['events']
          when 'U', 'UC', 'C' then
            if ygg_obj.changed?
              log.debug "#{ygg_obj.domain} changed"
              ygg_obj.store!
              ygg_obj.class.provision!
            end
          when 'D' then
            log.debug "#{ygg_obj.domain} deleted"
            ygg_obj.delete_stored!
            ygg_obj.class.provision!
          end
        else
          log.info "* Unhandled object type #{ygg_obj.class.name}"
        end
      rescue 
        ex = $!
        if defined?(Raven)
          Raven.extra_context(message: message, metadata: metadata.attributes) do
            Raven.capture_exception(ex)
          end
        else
          raise ex
        end
      end
    end

    def init(options = {})
      log.debug '+ Fetch service data'
      @sync = ::MailServerAgent::BulkSync.new
    end

    protected

    def load_ygg_obj(data)
      if data[:model]
        klass = data[:model].constantize
        obj = klass.new(data[:object])
      end
      obj
    rescue 
      log.error '* ' +  $!.to_s
      log.error '* ' +  $!.backtrace.join("\n* ")
      nil
    end
  end
end